/*
Module :IconComboBox.h
Purpose: Defines the interface for 3 MFC classes to implement icon selection combo boxes
Created: PJN / 2-08-1998

Copyright (c) 1998 - 2022 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/


//////////////////// Macros / Defines /////////////////////////////////////////

#pragma once

#ifndef __ICONCOMBOBOX_H__
#define __ICONCOMBOBOX_H__

#ifndef CICONCOMBOBOX_EXT_CLASS
#define CICONCOMBOBOX_EXT_CLASS
#endif //#ifndef CICONCOMBOBOX_EXT_CLASS

#ifndef CICONCOMBOBOX_EXT_API
#define CICONCOMBOBOX_EXT_API
#endif //#ifndef CICONCOMBOBOX_EXT_API


//////////////////// Includes /////////////////////////////////////////////////

class CICONCOMBOBOX_EXT_CLASS CIconComboBox : public CComboBox
{
public:
//Constructors / Destructors
  CIconComboBox() noexcept;
  CIconComboBox(const CIconComboBox&) = delete;
  CIconComboBox(CIconComboBox&&) = delete;
  ~CIconComboBox() = default; //NOLINT(modernize-use-override)

//Methods
  CIconComboBox& operator=(const CIconComboBox&) = delete;
  CIconComboBox& operator=(CIconComboBox&&) = delete;
  int AddIcon(_In_ LPCTSTR pszResourceID, _In_ HINSTANCE hInstance = AfxGetResourceHandle());
  int AddIcon(_In_ UINT nResourceID, _In_ HINSTANCE hInstance = AfxGetResourceHandle());
  int InsertIcon(_In_ int nIndex, _In_ LPCTSTR pszResourceID, _In_ HINSTANCE hInstance = AfxGetResourceHandle());
  int InsertIcon(_In_ int nIndex, _In_ UINT nResourceID, _In_ HINSTANCE hInstance = AfxGetResourceHandle());
  int DeleteIcon(_In_ int nIndex);
  _NODISCARD LPCTSTR GetCurSelIcon() const;
  _NODISCARD UINT GetCurSelIconAsInt() const;
  int SetCurSelIcon(_In_ UINT nResourceID);
  int SetCurSelIcon(_In_ LPCTSTR pszResourceID);
  int AddString(_In_ LPCTSTR pszString);
  int InsertString(_In_ int nIndex, _In_ LPCTSTR pszString);
  int DeleteString(_In_ int nIndex);
  void SetIconSize(_In_ int nCX, _In_ int nCY) noexcept;
  void SetUseSmallIconSize() noexcept;
  void SetUseLargeIconSize() noexcept;
  _NODISCARD int GetIconXSize() const noexcept { return m_nCXIcon; };
  _NODISCARD int GetIconYSize() const noexcept { return m_nCYIcon; };

protected:
//Member variables
  int m_nCXIcon;
  int m_nCYIcon;

//Methods
  void MeasureItem(LPMEASUREITEMSTRUCT lpMeasureItemStruct) override;
  void DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct) override;
  void PreSubclassWindow() override;
  void DeleteItem(LPDELETEITEMSTRUCT lpDeleteItemStruct) override;

  DECLARE_MESSAGE_MAP()

  DECLARE_DYNAMIC(CIconComboBox)
};

class CSmallIconComboBox : public CIconComboBox
{
public:
//Constructors / Destructors
  CSmallIconComboBox() noexcept;
  CSmallIconComboBox(const CSmallIconComboBox&) = delete;
  CSmallIconComboBox(CSmallIconComboBox&&) = delete;
  ~CSmallIconComboBox() = default; //NOLINT(modernize-use-override)

//Methods
  CSmallIconComboBox& operator=(const CSmallIconComboBox&) = delete;
  CSmallIconComboBox& operator=(CSmallIconComboBox&&) = delete;

protected:
  DECLARE_MESSAGE_MAP()

  DECLARE_DYNAMIC(CSmallIconComboBox)
};

class CICONCOMBOBOX_EXT_CLASS CLargeIconComboBox : public CIconComboBox
{
public:
//Constructors / Destructors
  CLargeIconComboBox() noexcept;
  CLargeIconComboBox(const CLargeIconComboBox&) = delete;
  CLargeIconComboBox(CLargeIconComboBox&&) = delete;
  ~CLargeIconComboBox() = default; //NOLINT(modernize-use-override)

//Methods
  CLargeIconComboBox& operator=(const CLargeIconComboBox&) = delete;
  CLargeIconComboBox& operator=(CLargeIconComboBox&&) = delete;

protected:
  DECLARE_MESSAGE_MAP()

  DECLARE_DYNAMIC(CLargeIconComboBox)
};

//DDX Support
void CICONCOMBOBOX_EXT_API DDX_CBIcon(_In_ CDataExchange* pDX, _In_ int nIDC, _Inout_ UINT& nResourceID);
void CICONCOMBOBOX_EXT_API DDX_CBIcon(_In_ CDataExchange* pDX, _In_ int nIDC, _Inout_ LPCTSTR& pszResourceID);

#endif //#ifndef __ICONCOMBOBOX_H__
